diff --git a/common/common.h b/common/common.h
index 574040d..9f37124 100644
--- a/common/common.h
+++ b/common/common.h
@@ -220,11 +220,18 @@ typedef struct
 
     int b_ref_pic_list_reordering_l0;
     int b_ref_pic_list_reordering_l1;
-    struct {
+    struct
+    {
         int idc;
         int arg;
     } ref_pic_list_order[2][16];
 
+    int i_mmco_idx;
+    struct // struct for future expansion
+    {
+        int i_difference_of_pic_nums;
+    } mmco[16];
+
     int i_cabac_init_idc;
 
     int i_qp;
@@ -385,6 +392,11 @@ struct x264_t
     x264_frame_t    *fref1[16+3];     /* ref list 1 */
     int             b_ref_reorder[2];
 
+    int             i_mmco_idx;
+    struct
+    {
+        int         i_difference_of_pic_nums;
+    } mmco[16];
 
 
     /* Current MB DCT coeffs */
diff --git a/encoder/encoder.c b/encoder/encoder.c
index af62765..f0d46ce 100644
--- a/encoder/encoder.c
+++ b/encoder/encoder.c
@@ -122,6 +122,14 @@ static void x264_slice_header_init( x264_t *h, x264_slice_header_t *sh,
 
     sh->b_ref_pic_list_reordering_l0 = h->b_ref_reorder[0];
     sh->b_ref_pic_list_reordering_l1 = h->b_ref_reorder[1];
+    
+    sh->i_mmco_idx = h->i_mmco_idx;
+    if( h->i_mmco_idx )
+    {
+        int j = 0;
+        for( ; j < h->i_mmco_idx; j++ )
+            sh->mmco[j].i_difference_of_pic_nums = h->mmco[j].i_difference_of_pic_nums;
+    }
 
     /* If the ref list isn't in the default order, construct reordering header */
     /* List1 reordering isn't needed yet */
@@ -272,7 +280,17 @@ static void x264_slice_header_write( bs_t *s, x264_slice_header_t *sh, int i_nal
         }
         else
         {
-            bs_write1( s, 0 );  /* adaptive_ref_pic_marking_mode_flag */
+            bs_write1( s, sh->i_mmco_idx > 0 ); /* adaptive_ref_pic_marking_mode_flag */
+            if( sh->i_mmco_idx > 0 )
+            {
+                int i = 0;
+                for( ; i < sh->i_mmco_idx; i++ )
+                {
+                    bs_write_ue( s, 1 ); /* mark short term ref as unused */
+                    bs_write_ue( s, sh->mmco[i].i_difference_of_pic_nums - 1 );
+                }
+                bs_write_ue( s, 0 );
+            }
         }
     }
 
@@ -1093,6 +1111,8 @@ static inline void x264_reference_build_list( x264_t *h, int i_poc )
                 h->b_ref_reorder[0] = 1;
                 break;
             }
+        if( h->param.b_bframe_pyramid )
+            h->b_ref_reorder[0] = 1;
     }
 
     h->i_ref1 = X264_MIN( h->i_ref1, h->frames.i_max_ref1 );
@@ -1227,6 +1247,42 @@ static inline void x264_reference_reset( x264_t *h )
     h->fenc->i_poc = 0;
 }
 
+static inline void x264_reference_partial_reset( x264_t *h )
+{
+    int refs = 0;
+    int lastnonbidx = -1;
+    while( h->frames.reference[refs] )
+    {
+        if( h->frames.reference[refs]->i_type == X264_TYPE_I
+         || h->frames.reference[refs]->i_type == X264_TYPE_IDR
+         || h->frames.reference[refs]->i_type == X264_TYPE_P )
+            lastnonbidx = refs;
+        refs++;
+    }
+    assert( lastnonbidx > -1 );
+    // Remove any brefs from dpb remembering if current frame is bref
+    // x264 has not added it yet. only will be true for i or P frames.
+    while( refs > lastnonbidx+1 )
+    {
+        int diff = h->i_frame_num - h->frames.reference[refs-1]->i_frame_num;
+        h->mmco[ h->i_mmco_idx++ ].i_difference_of_pic_nums = diff;
+        x264_frame_push_unused( h, x264_frame_pop( h->frames.reference ) );
+        refs--;
+    }
+    // This makes room in the dpb for the delayed display time of the later b-frame's
+    // the x264_max is to make sure that a foward and backward reference is always left
+    // for b-frames incase too few reference frames are used with b-pyramid. ( < 4 )
+    while( ( refs + 2 ) > X264_MAX( h->frames.i_max_dpb, 4 ) 
+                                && h->sh.i_type ==  SLICE_TYPE_B )
+    {
+        int diff = h->i_frame_num - h->frames.reference[0]->i_frame_num;
+        h->mmco[ h->i_mmco_idx++ ].i_difference_of_pic_nums = diff;
+        x264_frame_push_unused( h, x264_frame_shift( h->frames.reference ) );
+        lastnonbidx--;
+        refs--;
+    }
+}
+
 static inline void x264_slice_init( x264_t *h, int i_nal_type, int i_global_qp )
 {
     /* ------------------------ Create slice header  ----------------------- */
@@ -1705,6 +1761,7 @@ int     x264_encoder_encode( x264_t *h,
     {
         h->frames.i_last_idr = h->fenc->i_frame;
     }
+    h->i_mmco_idx = 0;
 
     /* ------------------- Setup frame context ----------------------------- */
     /* 5: Init data dependent of frame type */
@@ -1722,18 +1779,21 @@ int     x264_encoder_encode( x264_t *h,
         i_nal_type    = NAL_SLICE;
         i_nal_ref_idc = NAL_PRIORITY_HIGH; /* Not completely true but for now it is (as all I/P are kept as ref)*/
         h->sh.i_type = SLICE_TYPE_I;
+        x264_reference_partial_reset( h );
     }
     else if( h->fenc->i_type == X264_TYPE_P )
     {
         i_nal_type    = NAL_SLICE;
         i_nal_ref_idc = NAL_PRIORITY_HIGH; /* Not completely true but for now it is (as all I/P are kept as ref)*/
         h->sh.i_type = SLICE_TYPE_P;
+        x264_reference_partial_reset( h );
     }
     else if( h->fenc->i_type == X264_TYPE_BREF )
     {
         i_nal_type    = NAL_SLICE;
         i_nal_ref_idc = NAL_PRIORITY_HIGH; /* maybe add MMCO to forget it? -> low */
         h->sh.i_type = SLICE_TYPE_B;
+        x264_reference_partial_reset( h );
     }
     else    /* B frame */
     {